@@ -172,9 +172,9 @@ CHIPSET(0x46a0, adl_gt2, "ADL GT2", "Intel(R) Graphics")
 CHIPSET(0x46a1, adl_gt2, "ADL GT2", "Intel(R) Graphics")
 CHIPSET(0x46a2, adl_gt2, "ADL GT2", "Intel(R) Graphics")
 CHIPSET(0x46a3, adl_gt2, "ADL GT2", "Intel(R) Graphics")
-CHIPSET(0x46a6, adl_gt2, "ADL GT2", "Intel(R) Graphics")
-CHIPSET(0x46a8, adl_gt2, "ADL GT2", "Intel(R) Graphics")
-CHIPSET(0x46aa, adl_gt2, "ADL GT2", "Intel(R) Graphics")
+CHIPSET(0x46a6, adl_gt2, "ADL GT2", "Intel(R) Iris(R) Xe Graphics")
+CHIPSET(0x46a8, adl_gt2, "ADL GT2", "Intel(R) Iris(R) Xe Graphics")
+CHIPSET(0x46aa, adl_gt2, "ADL GT2", "Intel(R) Iris(R) Xe Graphics")
 CHIPSET(0x46b0, adl_gt2, "ADL GT2", "Intel(R) Graphics")
 CHIPSET(0x46b1, adl_gt2, "ADL GT2", "Intel(R) Graphics")
 CHIPSET(0x46b2, adl_gt2, "ADL GT2", "Intel(R) Graphics")
@@ -184,14 +184,14 @@ CHIPSET(0x46c1, adl_gt2, "ADL GT2", "Intel(R) Graphics")
 CHIPSET(0x46c2, adl_gt2, "ADL GT2", "Intel(R) Graphics")
 CHIPSET(0x46c3, adl_gt2, "ADL GT2", "Intel(R) Graphics")
 
-CHIPSET(0x46d0, adl_n, "ADL-N", "Intel(R) Graphics")
-CHIPSET(0x46d1, adl_n, "ADL-N", "Intel(R) Graphics")
-CHIPSET(0x46d2, adl_n, "ADL-N", "Intel(R) Graphics")
-CHIPSET(0x46d3, adl_n, "ADL-N", "Intel(R) Graphics")
-CHIPSET(0x46d4, adl_n, "ADL-N", "Intel(R) Graphics")
+CHIPSET(0x46d0, adl_gt1,  "ADL-N", "Intel(R) Graphics")
+CHIPSET(0x46d1, adl_gt1,  "ADL-N", "Intel(R) Graphics")
+CHIPSET(0x46d2, adl_gt05, "ADL-N", "Intel(R) Graphics")
+CHIPSET(0x46d3, adl_gt1,  "ADL-N", "Intel(R) Graphics")
+CHIPSET(0x46d4, adl_gt1,  "ADL-N", "Intel(R) Graphics")
 
-CHIPSET(0x9a40, tgl_gt2, "TGL GT2", "Intel(R) Xe Graphics")
-CHIPSET(0x9a49, tgl_gt2, "TGL GT2", "Intel(R) Xe Graphics")
+CHIPSET(0x9a40, tgl_gt2, "TGL GT2", "Intel(R) Iris(R) Xe Graphics")
+CHIPSET(0x9a49, tgl_gt2, "TGL GT2", "Intel(R) Iris(R) Xe Graphics")
 CHIPSET(0x9a59, tgl_gt2, "TGL GT2", "Intel(R) Graphics")
 CHIPSET(0x9a60, tgl_gt1, "TGL GT1", "Intel(R) UHD Graphics")
 CHIPSET(0x9a68, tgl_gt1, "TGL GT1", "Intel(R) UHD Graphics")
@@ -213,8 +213,8 @@ CHIPSET(0xa78b, rpl, "RPL-S", "Intel(R) Graphics")
 
 CHIPSET(0xa720, rpl_p, "RPL-P", "Intel(R) Graphics")
 CHIPSET(0xa721, rpl_p, "RPL-U", "Intel(R) Graphics")
-CHIPSET(0xa7a0, rpl_p, "RPL-P", "Intel(R) Graphics")
-CHIPSET(0xa7a1, rpl_p, "RPL-U", "Intel(R) Graphics")
+CHIPSET(0xa7a0, rpl_p, "RPL-P", "Intel(R) Iris(R) Xe Graphics")
+CHIPSET(0xa7a1, rpl_p, "RPL-U", "Intel(R) Iris(R) Xe Graphics")
 CHIPSET(0xa7a8, rpl_p, "RPL-P", "Intel(R) Graphics")
 CHIPSET(0xa7a9, rpl_p, "RPL-U", "Intel(R) Graphics")
 CHIPSET(0xa7aa, rpl_p, "RPL-P", "Intel(R) Graphics")
@@ -222,10 +222,10 @@ CHIPSET(0xa7ab, rpl_p, "RPL-P", "Intel(R) Graphics")
 CHIPSET(0xa7ac, rpl_p, "RPL-U", "Intel(R) Graphics")
 CHIPSET(0xa7ad, rpl_p, "RPL-U", "Intel(R) Graphics")
 
-CHIPSET(0x4905, dg1, "DG1", "Intel(R) Graphics")
+CHIPSET(0x4905, dg1, "DG1", "Intel(R) Iris(R) Xe MAX Graphics")
 CHIPSET(0x4906, dg1, "DG1", "Intel(R) Graphics")
 CHIPSET(0x4907, sg1, "SG1", "Intel(R) Graphics")
-CHIPSET(0x4908, dg1, "DG1", "Intel(R) Graphics")
+CHIPSET(0x4908, dg1, "DG1", "Intel(R) Iris(R) Xe Graphics")
 CHIPSET(0x4909, dg1, "DG1", "Intel(R) Graphics")
 
 CHIPSET(0x5690, dg2_g10, "DG2", "Intel(R) Arc(tm) A770M Graphics")
@@ -247,15 +247,47 @@ CHIPSET(0x56b0, dg2_g11, "DG2", "Intel(R) Arc(tm) Pro A30M Graphics")
 CHIPSET(0x56b1, dg2_g11, "DG2", "Intel(R) Arc(tm) Pro A40/A50 Graphics")
 CHIPSET(0x56b2, dg2_g12, "DG2", "Intel(R) Arc(tm) Pro A60M Graphics")
 CHIPSET(0x56b3, dg2_g12, "DG2", "Intel(R) Arc(tm) Pro A60 Graphics")
-CHIPSET(0x56ba, dg2_g11, "DG2", "Intel(R) Graphics")
-CHIPSET(0x56bb, dg2_g11, "DG2", "Intel(R) Graphics")
-CHIPSET(0x56bc, dg2_g11, "DG2", "Intel(R) Graphics")
-CHIPSET(0x56bd, dg2_g11, "DG2", "Intel(R) Graphics")
-CHIPSET(0x56c0, atsm_g10, "ATS-M", "Intel(R) Data Center GPU Flex Series 170 Graphics")
-CHIPSET(0x56c1, atsm_g11, "ATS-M", "Intel(R) Data Center GPU Flex Series 140 Graphics")
+CHIPSET(0x56ba, dg2_g11, "DG2", "Intel(R) Arc(tm) A380E Graphics")
+CHIPSET(0x56bb, dg2_g11, "DG2", "Intel(R) Arc(tm) A310E Graphics")
+CHIPSET(0x56bc, dg2_g11, "DG2", "Intel(R) Arc(tm) A370E Graphics")
+CHIPSET(0x56bd, dg2_g11, "DG2", "Intel(R) Arc(tm) A350E Graphics")
+CHIPSET(0x56be, dg2_g10, "DG2", "Intel(R) Arc(tm) A750E Graphics")
+CHIPSET(0x56bf, dg2_g10, "DG2", "Intel(R) Arc(tm) A580E Graphics")
+CHIPSET(0x56c0, atsm_g10, "ATS-M", "Intel(R) Data Center GPU Flex 170")
+CHIPSET(0x56c1, atsm_g11, "ATS-M", "Intel(R) Data Center GPU Flex 140")
+CHIPSET(0x56c2, atsm_g10, "ATS-M", "Intel(R) Data Center GPU Flex 170V")
+
+CHIPSET(0x6420, lnl, "LNL", "Intel(R) Graphics")
+CHIPSET(0x64a0, lnl, "LNL", "Intel(R) Graphics")
+CHIPSET(0x64b0, lnl, "LNL", "Intel(R) Graphics")
 
 CHIPSET(0x7d40, mtl_u, "MTL", "Intel(R) Graphics")
-CHIPSET(0x7d45, mtl_h, "MTL", "Intel(R) Graphics")
+CHIPSET(0x7d45, mtl_u, "MTL", "Intel(R) Graphics")
 CHIPSET(0x7d55, mtl_h, "MTL", "Intel(R) Arc(tm) Graphics")
 CHIPSET(0x7d60, mtl_u, "MTL", "Intel(R) Graphics")
 CHIPSET(0x7dd5, mtl_h, "MTL", "Intel(R) Graphics")
+
+CHIPSET(0x7d41, arl_u, "ARL", "Intel(R) Graphics")
+CHIPSET(0x7d51, arl_h, "ARL", "Intel(R) Graphics")
+CHIPSET(0x7d67, arl_u, "ARL", "Intel(R) Graphics")
+CHIPSET(0x7dd1, arl_h, "ARL", "Intel(R) Graphics")
+CHIPSET(0xb640, arl_u, "ARL", "Intel(R) Graphics")
+
+CHIPSET(0xe202, bmg, "BMG G21", "Intel(R) Graphics")
+CHIPSET(0xe20b, bmg, "BMG G21", "Intel(R) Graphics")
+CHIPSET(0xe20c, bmg, "BMG G21", "Intel(R) Graphics")
+CHIPSET(0xe20d, bmg, "BMG G21", "Intel(R) Graphics")
+CHIPSET(0xe210, bmg, "BMG G21", "Intel(R) Graphics")
+CHIPSET(0xe211, bmg, "BMG G21", "Intel(R) Graphics")
+CHIPSET(0xe212, bmg, "BMG G21", "Intel(R) Graphics")
+CHIPSET(0xe215, bmg, "BMG", "Intel(R) Graphics")
+CHIPSET(0xe216, bmg, "BMG", "Intel(R) Graphics")
+
+CHIPSET(0xb080, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
+CHIPSET(0xb081, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
+CHIPSET(0xb082, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
+CHIPSET(0xb083, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
+CHIPSET(0xb08f, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
+CHIPSET(0xb090, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
+CHIPSET(0xb0a0, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
+CHIPSET(0xb0b0, ptl, "PTL", "Intel(R) Graphics", FORCE_PROBE)
