@@ -7,8 +7,8 @@
 #include <assert.h>
 #include <stdbool.h>
 
+#include "nir/radv_meta_nir.h"
 #include "radv_meta.h"
-#include "sid.h"
 
 enum radv_color_op {
    FAST_CLEAR_ELIMINATE,
@@ -16,43 +16,6 @@ enum radv_color_op {
    DCC_DECOMPRESS,
 };
 
-static nir_shader *
-build_dcc_decompress_compute_shader(struct radv_device *dev)
-{
-   const struct glsl_type *img_type = glsl_image_type(GLSL_SAMPLER_DIM_2D, false, GLSL_TYPE_FLOAT);
-
-   nir_builder b = radv_meta_init_shader(dev, MESA_SHADER_COMPUTE, "dcc_decompress_compute");
-
-   /* We need at least 16/16/1 to cover an entire DCC block in a single workgroup. */
-   b.shader->info.workgroup_size[0] = 16;
-   b.shader->info.workgroup_size[1] = 16;
-   nir_variable *input_img = nir_variable_create(b.shader, nir_var_image, img_type, "in_img");
-   input_img->data.descriptor_set = 0;
-   input_img->data.binding = 0;
-
-   nir_variable *output_img = nir_variable_create(b.shader, nir_var_image, img_type, "out_img");
-   output_img->data.descriptor_set = 0;
-   output_img->data.binding = 1;
-
-   nir_def *global_id = get_global_ids(&b, 2);
-   nir_def *img_coord = nir_vec4(&b, nir_channel(&b, global_id, 0), nir_channel(&b, global_id, 1), nir_undef(&b, 1, 32),
-                                 nir_undef(&b, 1, 32));
-
-   nir_def *data = nir_image_deref_load(&b, 4, 32, &nir_build_deref_var(&b, input_img)->def, img_coord,
-                                        nir_undef(&b, 1, 32), nir_imm_int(&b, 0), .image_dim = GLSL_SAMPLER_DIM_2D);
-
-   /* We need a SCOPE_DEVICE memory_scope because ACO will avoid
-    * creating a vmcnt(0) because it expects the L1 cache to keep memory
-    * operations in-order for the same workgroup. The vmcnt(0) seems
-    * necessary however. */
-   nir_barrier(&b, .execution_scope = SCOPE_WORKGROUP, .memory_scope = SCOPE_DEVICE,
-               .memory_semantics = NIR_MEMORY_ACQ_REL, .memory_modes = nir_var_mem_ssbo);
-
-   nir_image_deref_store(&b, &nir_build_deref_var(&b, output_img)->def, img_coord, nir_undef(&b, 1, 32), data,
-                         nir_imm_int(&b, 0), .image_dim = GLSL_SAMPLER_DIM_2D);
-   return b.shader;
-}
-
 static VkResult
 get_dcc_decompress_compute_pipeline(struct radv_device *device, VkPipeline *pipeline_out, VkPipelineLayout *layout_out)
 {
@@ -92,7 +55,7 @@ get_dcc_decompress_compute_pipeline(struct radv_device *device, VkPipeline *pipe
       return VK_SUCCESS;
    }
 
-   nir_shader *cs = build_dcc_decompress_compute_shader(device);
+   nir_shader *cs = radv_meta_nir_build_dcc_decompress_compute_shader(device);
 
    const VkPipelineShaderStageCreateInfo stage_info = {
       .sType = VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO,
@@ -145,8 +108,8 @@ get_pipeline(struct radv_device *device, enum radv_color_op op, VkPipeline *pipe
       return VK_SUCCESS;
    }
 
-   nir_shader *vs_module = radv_meta_build_nir_vs_generate_vertices(device);
-   nir_shader *fs_module = radv_meta_build_nir_fs_noop(device);
+   nir_shader *vs_module = radv_meta_nir_build_vs_generate_vertices(device);
+   nir_shader *fs_module = radv_meta_nir_build_fs_noop(device);
 
    VkGraphicsPipelineCreateInfoRADV radv_info = {
       .sType = VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_CREATE_INFO_RADV,
@@ -268,14 +231,14 @@ radv_emit_set_predication_state_from_image(struct radv_cmd_buffer *cmd_buffer, s
    uint64_t va = 0;
 
    if (value)
-      va = radv_image_get_va(image, 0) + pred_offset;
+      va = image->bindings[0].addr + pred_offset;
 
    radv_emit_set_predication_state(cmd_buffer, true, PREDICATION_OP_BOOL64, va);
 }
 
 static void
 radv_process_color_image_layer(struct radv_cmd_buffer *cmd_buffer, struct radv_image *image,
-                               const VkImageSubresourceRange *range, int level, int layer, bool flush_cb)
+                               const VkImageSubresourceRange *range, int level, int layer, enum radv_color_op op)
 {
    struct radv_device *device = radv_cmd_buffer_device(cmd_buffer);
    struct radv_image_view iview;
@@ -284,22 +247,28 @@ radv_process_color_image_layer(struct radv_cmd_buffer *cmd_buffer, struct radv_i
    width = u_minify(image->vk.extent.width, range->baseMipLevel + level);
    height = u_minify(image->vk.extent.height, range->baseMipLevel + level);
 
-   radv_image_view_init(&iview, device,
-                        &(VkImageViewCreateInfo){
-                           .sType = VK_STRUCTURE_TYPE_IMAGE_VIEW_CREATE_INFO,
-                           .image = radv_image_to_handle(image),
-                           .viewType = radv_meta_get_view_type(image),
-                           .format = image->vk.format,
-                           .subresourceRange =
-                              {
-                                 .aspectMask = VK_IMAGE_ASPECT_COLOR_BIT,
-                                 .baseMipLevel = range->baseMipLevel + level,
-                                 .levelCount = 1,
-                                 .baseArrayLayer = range->baseArrayLayer + layer,
-                                 .layerCount = 1,
-                              },
-                        },
-                        NULL);
+   /* TC-compatible CMASK must be disabled (ie. FMASK_COMPRESS_1FRAG_ONLY must be 0), otherwise the
+    * FMASK decompress operation doesn't happen (DCC_DECOMPRESS implicitly uses FMASK_DECOMPRESS).
+    */
+   const bool disable_tc_compat_cmask_mrt = op == FMASK_DECOMPRESS || op == DCC_DECOMPRESS;
+
+   radv_image_view_init(
+      &iview, device,
+      &(VkImageViewCreateInfo){
+         .sType = VK_STRUCTURE_TYPE_IMAGE_VIEW_CREATE_INFO,
+         .image = radv_image_to_handle(image),
+         .viewType = radv_meta_get_view_type(image),
+         .format = image->vk.format,
+         .subresourceRange =
+            {
+               .aspectMask = VK_IMAGE_ASPECT_COLOR_BIT,
+               .baseMipLevel = range->baseMipLevel + level,
+               .levelCount = 1,
+               .baseArrayLayer = range->baseArrayLayer + layer,
+               .layerCount = 1,
+            },
+      },
+      &(struct radv_image_view_extra_create_info){.disable_tc_compat_cmask_mrt = disable_tc_compat_cmask_mrt});
 
    const VkRenderingAttachmentInfo color_att = {
       .sType = VK_STRUCTURE_TYPE_RENDERING_ATTACHMENT_INFO,
@@ -320,13 +289,14 @@ radv_process_color_image_layer(struct radv_cmd_buffer *cmd_buffer, struct radv_i
 
    radv_CmdBeginRendering(radv_cmd_buffer_to_handle(cmd_buffer), &rendering_info);
 
-   if (flush_cb)
+   /* Flushing CB is required before/after {FMASK,DCC}_DECOMPRESS. */
+   if (op == FMASK_DECOMPRESS || op == DCC_DECOMPRESS)
       cmd_buffer->state.flush_bits |= radv_dst_access_flush(cmd_buffer, VK_PIPELINE_STAGE_2_ALL_COMMANDS_BIT,
                                                             VK_ACCESS_2_COLOR_ATTACHMENT_READ_BIT, 0, image, range);
 
    radv_CmdDraw(radv_cmd_buffer_to_handle(cmd_buffer), 3, 1, 0, 0);
 
-   if (flush_cb)
+   if (op == FMASK_DECOMPRESS || op == DCC_DECOMPRESS)
       cmd_buffer->state.flush_bits |= radv_src_access_flush(cmd_buffer, VK_PIPELINE_STAGE_2_ALL_COMMANDS_BIT,
                                                             VK_ACCESS_2_COLOR_ATTACHMENT_WRITE_BIT, 0, image, range);
 
@@ -342,7 +312,6 @@ radv_process_color_image(struct radv_cmd_buffer *cmd_buffer, struct radv_image *
    struct radv_device *device = radv_cmd_buffer_device(cmd_buffer);
    struct radv_meta_saved_state saved_state;
    bool old_predicating = false;
-   bool flush_cb = false;
    uint64_t pred_offset;
    VkPipelineLayout layout;
    VkPipeline pipeline;
@@ -360,15 +329,9 @@ radv_process_color_image(struct radv_cmd_buffer *cmd_buffer, struct radv_image *
       break;
    case FMASK_DECOMPRESS:
       pred_offset = 0; /* FMASK_DECOMPRESS is never predicated. */
-
-      /* Flushing CB is required before and after FMASK_DECOMPRESS. */
-      flush_cb = true;
       break;
    case DCC_DECOMPRESS:
       pred_offset = image->dcc_pred_offset;
-
-      /* Flushing CB is required before and after DCC_DECOMPRESS. */
-      flush_cb = true;
       break;
    default:
       unreachable("Invalid color op");
@@ -418,7 +381,7 @@ radv_process_color_image(struct radv_cmd_buffer *cmd_buffer, struct radv_image *
                          });
 
       for (uint32_t s = 0; s < vk_image_subresource_layer_count(&image->vk, subresourceRange); s++) {
-         radv_process_color_image_layer(cmd_buffer, image, subresourceRange, l, s, flush_cb);
+         radv_process_color_image_layer(cmd_buffer, image, subresourceRange, l, s, op);
       }
    }
 
@@ -450,52 +413,36 @@ radv_process_color_image(struct radv_cmd_buffer *cmd_buffer, struct radv_image *
       radv_update_dcc_metadata(cmd_buffer, image, subresourceRange, false);
 }
 
-static void
+void
 radv_fast_clear_eliminate(struct radv_cmd_buffer *cmd_buffer, struct radv_image *image,
                           const VkImageSubresourceRange *subresourceRange)
 {
    struct radv_barrier_data barrier = {0};
 
+   if (cmd_buffer->qf != RADV_QUEUE_GENERAL)
+      return;
+
    barrier.layout_transitions.fast_clear_eliminate = 1;
    radv_describe_layout_transition(cmd_buffer, &barrier);
 
    radv_process_color_image(cmd_buffer, image, subresourceRange, FAST_CLEAR_ELIMINATE);
 }
 
-static void
+void
 radv_fmask_decompress(struct radv_cmd_buffer *cmd_buffer, struct radv_image *image,
                       const VkImageSubresourceRange *subresourceRange)
 {
    struct radv_barrier_data barrier = {0};
 
+   if (cmd_buffer->qf != RADV_QUEUE_GENERAL)
+      return;
+
    barrier.layout_transitions.fmask_decompress = 1;
    radv_describe_layout_transition(cmd_buffer, &barrier);
 
    radv_process_color_image(cmd_buffer, image, subresourceRange, FMASK_DECOMPRESS);
 }
 
-void
-radv_fast_clear_flush_image_inplace(struct radv_cmd_buffer *cmd_buffer, struct radv_image *image,
-                                    const VkImageSubresourceRange *subresourceRange)
-{
-   if (radv_image_has_fmask(image) && !image->tc_compatible_cmask) {
-      if (radv_image_has_dcc(image) && radv_image_has_cmask(image)) {
-         /* MSAA images with DCC and CMASK might have been fast-cleared and might require a FCE but
-          * FMASK_DECOMPRESS can't eliminate DCC fast clears.
-          */
-         radv_fast_clear_eliminate(cmd_buffer, image, subresourceRange);
-      }
-
-      radv_fmask_decompress(cmd_buffer, image, subresourceRange);
-   } else {
-      /* Skip fast clear eliminate for images that support comp-to-single fast clears. */
-      if (image->support_comp_to_single)
-         return;
-
-      radv_fast_clear_eliminate(cmd_buffer, image, subresourceRange);
-   }
-}
-
 static void
 radv_decompress_dcc_compute(struct radv_cmd_buffer *cmd_buffer, struct radv_image *image,
                             const VkImageSubresourceRange *subresourceRange)
@@ -559,33 +506,27 @@ radv_decompress_dcc_compute(struct radv_cmd_buffer *cmd_buffer, struct radv_imag
                               },
                               &(struct radv_image_view_extra_create_info){.disable_compression = true});
 
-         radv_meta_push_descriptor_set(
-            cmd_buffer, VK_PIPELINE_BIND_POINT_COMPUTE, layout, 0, 2,
-            (VkWriteDescriptorSet[]){{.sType = VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET,
-                                      .dstBinding = 0,
-                                      .dstArrayElement = 0,
-                                      .descriptorCount = 1,
-                                      .descriptorType = VK_DESCRIPTOR_TYPE_STORAGE_IMAGE,
-                                      .pImageInfo =
-                                         (VkDescriptorImageInfo[]){
-                                            {
-                                               .sampler = VK_NULL_HANDLE,
-                                               .imageView = radv_image_view_to_handle(&load_iview),
-                                               .imageLayout = VK_IMAGE_LAYOUT_GENERAL,
-                                            },
-                                         }},
-                                     {.sType = VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET,
-                                      .dstBinding = 1,
-                                      .dstArrayElement = 0,
-                                      .descriptorCount = 1,
-                                      .descriptorType = VK_DESCRIPTOR_TYPE_STORAGE_IMAGE,
-                                      .pImageInfo = (VkDescriptorImageInfo[]){
-                                         {
-                                            .sampler = VK_NULL_HANDLE,
-                                            .imageView = radv_image_view_to_handle(&store_iview),
-                                            .imageLayout = VK_IMAGE_LAYOUT_GENERAL,
-                                         },
-                                      }}});
+         radv_meta_bind_descriptors(
+            cmd_buffer, VK_PIPELINE_BIND_POINT_COMPUTE, layout, 2,
+            (VkDescriptorGetInfoEXT[]){{.sType = VK_STRUCTURE_TYPE_DESCRIPTOR_GET_INFO_EXT,
+                                        .type = VK_DESCRIPTOR_TYPE_STORAGE_IMAGE,
+                                        .data.pStorageImage =
+                                           (VkDescriptorImageInfo[]){
+                                              {
+                                                 .sampler = VK_NULL_HANDLE,
+                                                 .imageView = radv_image_view_to_handle(&load_iview),
+                                                 .imageLayout = VK_IMAGE_LAYOUT_GENERAL,
+                                              },
+                                           }},
+                                       {.sType = VK_STRUCTURE_TYPE_DESCRIPTOR_GET_INFO_EXT,
+                                        .type = VK_DESCRIPTOR_TYPE_STORAGE_IMAGE,
+                                        .data.pStorageImage = (VkDescriptorImageInfo[]){
+                                           {
+                                              .sampler = VK_NULL_HANDLE,
+                                              .imageView = radv_image_view_to_handle(&store_iview),
+                                              .imageLayout = VK_IMAGE_LAYOUT_GENERAL,
+                                           },
+                                        }}});
 
          radv_unaligned_dispatch(cmd_buffer, width, height, 1);
 
@@ -616,8 +557,10 @@ radv_decompress_dcc(struct radv_cmd_buffer *cmd_buffer, struct radv_image *image
    barrier.layout_transitions.dcc_decompress = 1;
    radv_describe_layout_transition(cmd_buffer, &barrier);
 
-   if (cmd_buffer->qf == RADV_QUEUE_GENERAL)
+   if (cmd_buffer->qf == RADV_QUEUE_GENERAL) {
       radv_process_color_image(cmd_buffer, image, subresourceRange, DCC_DECOMPRESS);
-   else
+   } else {
+      assert(cmd_buffer->qf == RADV_QUEUE_COMPUTE);
       radv_decompress_dcc_compute(cmd_buffer, image, subresourceRange);
+   }
 }
