@@ -99,7 +99,9 @@ enum radv_cmd_dirty_bits {
    RADV_CMD_DIRTY_TASK_STATE = 1ull << 15,
    RADV_CMD_DIRTY_DEPTH_STENCIL_STATE = 1ull << 16,
    RADV_CMD_DIRTY_RASTER_STATE = 1ull << 17,
-   RADV_CMD_DIRTY_ALL = (1ull << 18) - 1,
+   RADV_CMD_DIRTY_MSAA_STATE = 1ull << 18,
+   RADV_CMD_DIRTY_CLIP_RECTS_STATE = 1ull << 19,
+   RADV_CMD_DIRTY_ALL = (1ull << 20) - 1,
 
    RADV_CMD_DIRTY_SHADER_QUERY = RADV_CMD_DIRTY_NGG_STATE | RADV_CMD_DIRTY_TASK_STATE,
 };
@@ -144,14 +146,13 @@ enum radv_cmd_flush_bits {
 };
 
 struct radv_vertex_binding {
-   VkDeviceSize offset;
+   uint64_t addr;
    VkDeviceSize size;
    VkDeviceSize stride;
 };
 
 struct radv_streamout_binding {
-   struct radv_buffer *buffer;
-   VkDeviceSize offset;
+   uint64_t va;
    VkDeviceSize size;
 };
 
@@ -277,6 +278,7 @@ enum radv_tracked_reg {
 
    RADV_TRACKED_PA_SC_BINNER_CNTL_0,
    RADV_TRACKED_PA_SC_SHADER_CONTROL,
+   RADV_TRACKED_PA_SC_HISZ_CONTROL = RADV_TRACKED_PA_SC_SHADER_CONTROL, /* GFX12 (slot reused) */
    RADV_TRACKED_PA_SC_LINE_CNTL,
    RADV_TRACKED_PA_SC_LINE_STIPPLE,
    RADV_TRACKED_PA_SC_LINE_STIPPLE_RESET, /* GFX12 */
@@ -330,6 +332,18 @@ enum radv_tracked_reg {
    RADV_TRACKED_PA_SU_LINE_CNTL,
    RADV_TRACKED_PA_SU_SC_MODE_CNTL,
 
+   /* 2 consecutive registers */
+   RADV_TRACKED_PA_SC_AA_MASK_X0Y0_X1Y0,
+
+   RADV_TRACKED_DB_EQAA,
+   RADV_TRACKED_DB_ALPHA_TO_MASK,
+   RADV_TRACKED_PA_SC_CONSERVATIVE_RASTERIZATION_CNTL, /* GFX9+ */
+   RADV_TRACKED_PA_SC_AA_CONFIG,
+   RADV_TRACKED_PA_SC_MODE_CNTL_0,
+   RADV_TRACKED_PA_SC_SAMPLE_PROPERTIES, /* GFX12+ */
+
+   RADV_TRACKED_DB_RENDER_OVERRIDE, /* GFX12+ */
+
    RADV_NUM_ALL_TRACKED_REGS,
 };
 
@@ -408,6 +422,8 @@ struct radv_cmd_state {
    uint64_t predication_va;
    uint64_t mec_inv_pred_va;  /* For inverted predication when using MEC. */
    bool mec_inv_pred_emitted; /* To ensure we don't have to repeat inverting the VA. */
+   bool saved_user_cond_render;
+   bool is_user_cond_render_suspended;
 
    /* Inheritance info. */
    VkQueryPipelineStatisticFlags inherited_pipeline_statistics;
@@ -480,6 +496,7 @@ struct radv_cmd_state {
    uint8_t vtx_emit_num;
    bool uses_drawid;
    bool uses_baseinstance;
+   bool can_use_simple_vertex_input;
 
    bool uses_out_of_order_rast;
    bool uses_vrs;
@@ -492,7 +509,7 @@ struct radv_cmd_state {
 };
 
 struct radv_enc_state {
-   uint32_t task_size_offset;
+   uint32_t *p_task_size;
    uint32_t total_task_size;
    unsigned shifter;
    unsigned bits_in_shifter;
@@ -522,15 +539,12 @@ struct radv_cmd_buffer {
    VkCommandBufferUsageFlags usage_flags;
    struct radeon_cmdbuf *cs;
    struct radv_cmd_state state;
-   struct radv_buffer *vertex_binding_buffers[MAX_VBS];
    struct radv_vertex_binding vertex_bindings[MAX_VBS];
-   uint32_t used_vertex_bindings;
    struct radv_streamout_binding streamout_bindings[MAX_SO_BUFFERS];
    enum radv_queue_family qf;
 
    uint8_t push_constants[MAX_PUSH_CONSTANTS_SIZE];
    VkShaderStageFlags push_constant_stages;
-   struct radv_descriptor_set_header meta_push_descriptors;
 
    struct radv_descriptor_state descriptors[MAX_BIND_POINTS];
 
@@ -673,39 +687,6 @@ radv_get_num_pipeline_stat_queries(struct radv_cmd_buffer *cmd_buffer)
           cmd_buffer->state.active_prims_xfb_queries;
 }
 
-static inline void
-radv_emit_shader_pointer_head(struct radeon_cmdbuf *cs, unsigned sh_offset, unsigned pointer_count,
-                              bool use_32bit_pointers)
-{
-   radeon_emit(cs, PKT3(PKT3_SET_SH_REG, pointer_count * (use_32bit_pointers ? 1 : 2), 0));
-   radeon_emit(cs, (sh_offset - SI_SH_REG_OFFSET) >> 2);
-}
-
-static inline void
-radv_emit_shader_pointer_body(const struct radv_device *device, struct radeon_cmdbuf *cs, uint64_t va,
-                              bool use_32bit_pointers)
-{
-   const struct radv_physical_device *pdev = radv_device_physical(device);
-
-   radeon_emit(cs, va);
-
-   if (use_32bit_pointers) {
-      assert(va == 0 || (va >> 32) == pdev->info.address32_hi);
-   } else {
-      radeon_emit(cs, va >> 32);
-   }
-}
-
-static inline void
-radv_emit_shader_pointer(const struct radv_device *device, struct radeon_cmdbuf *cs, uint32_t sh_offset, uint64_t va,
-                         bool global)
-{
-   bool use_32bit_pointers = !global;
-
-   radv_emit_shader_pointer_head(cs, sh_offset, 1, use_32bit_pointers);
-   radv_emit_shader_pointer_body(device, cs, va, use_32bit_pointers);
-}
-
 bool radv_cmd_buffer_uses_mec(struct radv_cmd_buffer *cmd_buffer);
 
 void radv_cmd_buffer_reset_rendering(struct radv_cmd_buffer *cmd_buffer);
@@ -760,10 +741,6 @@ struct radv_resolve_barrier {
 
 void radv_emit_resolve_barrier(struct radv_cmd_buffer *cmd_buffer, const struct radv_resolve_barrier *barrier);
 
-void radv_meta_push_descriptor_set(struct radv_cmd_buffer *cmd_buffer, VkPipelineBindPoint pipelineBindPoint,
-                                   VkPipelineLayout _layout, uint32_t set, uint32_t descriptorWriteCount,
-                                   const VkWriteDescriptorSet *pDescriptorWrites);
-
 struct radv_dispatch_info {
    /**
     * Determine the layout of the grid (in block units) to be used.
@@ -787,10 +764,9 @@ struct radv_dispatch_info {
    bool ordered;
 
    /**
-    * Indirect compute parameters resource.
+    * Indirect compute parameters VA.
     */
-   struct radeon_winsys_bo *indirect;
-   uint64_t va;
+   uint64_t indirect_va;
 };
 
 void radv_compute_dispatch(struct radv_cmd_buffer *cmd_buffer, const struct radv_dispatch_info *info);
@@ -803,8 +779,6 @@ void radv_compute_dispatch(struct radv_cmd_buffer *cmd_buffer, const struct radv
  */
 void radv_unaligned_dispatch(struct radv_cmd_buffer *cmd_buffer, uint32_t x, uint32_t y, uint32_t z);
 
-void radv_indirect_dispatch(struct radv_cmd_buffer *cmd_buffer, struct radeon_winsys_bo *bo, uint64_t va);
-
 uint32_t radv_init_fmask(struct radv_cmd_buffer *cmd_buffer, struct radv_image *image,
                          const VkImageSubresourceRange *range);
 
@@ -844,4 +818,23 @@ void radv_emit_compute_shader(const struct radv_physical_device *pdev, struct ra
 void radv_upload_indirect_descriptor_sets(struct radv_cmd_buffer *cmd_buffer,
                                           struct radv_descriptor_state *descriptors_state);
 
+static inline void
+radv_suspend_conditional_rendering(struct radv_cmd_buffer *cmd_buffer)
+{
+   assert(!cmd_buffer->state.is_user_cond_render_suspended);
+
+   cmd_buffer->state.saved_user_cond_render = cmd_buffer->state.predicating;
+   cmd_buffer->state.predicating = false;
+   cmd_buffer->state.is_user_cond_render_suspended = true;
+}
+
+static inline void
+radv_resume_conditional_rendering(struct radv_cmd_buffer *cmd_buffer)
+{
+   assert(cmd_buffer->state.is_user_cond_render_suspended);
+
+   cmd_buffer->state.predicating = cmd_buffer->state.saved_user_cond_render;
+   cmd_buffer->state.is_user_cond_render_suspended = false;
+}
+
 #endif /* RADV_CMD_BUFFER_H */
