@@ -168,7 +168,7 @@ radv_CreateDescriptorSetLayout(VkDevice _device, const VkDescriptorSetLayoutCrea
     * they are reference counted and may not be destroyed when vkDestroyDescriptorSetLayout is
     * called.
     */
-   set_layout = vk_descriptor_set_layout_zalloc(&device->vk, size);
+   set_layout = vk_descriptor_set_layout_zalloc(&device->vk, size, pCreateInfo);
    if (!set_layout)
       return vk_error(device, VK_ERROR_OUT_OF_HOST_MEMORY);
 
@@ -1090,7 +1090,7 @@ write_buffer_descriptor_impl(struct radv_device *device, struct radv_cmd_buffer
    uint64_t va = 0, range = 0;
 
    if (buffer) {
-      va = radv_buffer_get_va(buffer->bo) + buffer_info->offset + buffer->offset;
+      va = vk_buffer_address(&buffer->vk, buffer_info->offset);
 
       range = vk_buffer_range(&buffer->vk, buffer_info->offset, buffer_info->range);
       assert(buffer->vk.size > 0 && range > 0);
@@ -1128,7 +1128,6 @@ write_dynamic_buffer_descriptor(struct radv_device *device, struct radv_descript
                                 struct radeon_winsys_bo **buffer_list, const VkDescriptorBufferInfo *buffer_info)
 {
    VK_FROM_HANDLE(radv_buffer, buffer, buffer_info->buffer);
-   uint64_t va;
    unsigned size;
 
    if (!buffer) {
@@ -1137,8 +1136,6 @@ write_dynamic_buffer_descriptor(struct radv_device *device, struct radv_descript
       return;
    }
 
-   va = radv_buffer_get_va(buffer->bo);
-
    size = vk_buffer_range(&buffer->vk, buffer_info->offset, buffer_info->range);
    assert(buffer->vk.size > 0 && size > 0);
 
@@ -1148,8 +1145,7 @@ write_dynamic_buffer_descriptor(struct radv_device *device, struct radv_descript
     * buffer accesses. */
    size = align(size, 4);
 
-   va += buffer_info->offset + buffer->offset;
-   range->va = va;
+   range->va = vk_buffer_address(&buffer->vk, buffer_info->offset);
    range->size = size;
 
    *buffer_list = buffer->bo;
