@@ -11,6 +11,7 @@
 #include "tu_device.h"
 #include "tu_event.h"
 #include "tu_image.h"
+#include "tu_queue.h"
 #include "tu_query_pool.h"
 
 #include <cstdio>
@@ -278,7 +279,7 @@ tu_rmv_log_buffer_bind(struct tu_device *device, struct tu_buffer *buffer)
 
    tu_rmv_emit_resource_bind_locked(device,
                                     tu_rmv_get_resource_id_locked(device, buffer),
-                                    buffer->bo ? buffer->iova : 0,
+                                    buffer->bo ? buffer->vk.device_address : 0,
                                     buffer->vk.size);
 
    simple_mtx_unlock(&device->vk.memory_trace_data.token_mtx);
@@ -513,10 +514,8 @@ tu_rmv_log_event_create(struct tu_device *device,
    vk_rmv_emit_token(&device->vk.memory_trace_data,
                      VK_RMV_TOKEN_TYPE_RESOURCE_CREATE, &token);
 
-   if (event->bo) {
-      tu_rmv_emit_resource_bind_locked(device, token.resource_id,
-                                       event->bo->iova, event->bo->size);
-   }
+   tu_rmv_emit_resource_bind_locked(device, token.resource_id,
+                                    event->bo.iova, event->bo.size);
 
    simple_mtx_unlock(&device->vk.memory_trace_data.token_mtx);
 }
