@@ -379,6 +379,7 @@ struct iris_cs_data {
    enum intel_compute_walk_order walk_order;
 
    bool uses_barrier;
+   bool uses_sampler;
    bool first_param_is_builtin_subgroup_id;
 };
 
@@ -499,6 +500,7 @@ enum pipe_control_flags
    PIPE_CONTROL_L3_READ_ONLY_CACHE_INVALIDATE   = (1 << 28),
    PIPE_CONTROL_UNTYPED_DATAPORT_CACHE_FLUSH    = (1 << 29),
    PIPE_CONTROL_CCS_CACHE_FLUSH                 = (1 << 30),
+   PIPE_CONTROL_L3_FABRIC_FLUSH                 = (1 << 31),
 };
 
 #define PIPE_CONTROL_CACHE_FLUSH_BITS \
@@ -588,9 +590,6 @@ struct iris_uncompiled_shader {
    /* Whether shader uses atomic operations. */
    bool uses_atomic_load_store;
 
-   /** Size (in bytes) of the kernel input data */
-   unsigned kernel_input_size;
-
    /** Size (in bytes) of the local (shared) data passed as kernel inputs */
    unsigned kernel_shared_size;
 
@@ -682,9 +681,6 @@ struct iris_compiled_shader {
    uint32_t *system_values;
    unsigned num_system_values;
 
-   /** Size (in bytes) of the kernel input data */
-   unsigned kernel_input_size;
-
    /** Number of constbufs expected by the shader. */
    unsigned num_cbufs;
 
@@ -1012,6 +1008,8 @@ struct iris_context {
       struct pipe_stencil_ref stencil_ref;
       struct pipe_framebuffer_state framebuffer;
       struct pipe_clip_state clip_planes;
+      /* width and height treated like x2 and y2 */
+      struct pipe_box render_area;
 
       float default_outer_level[4];
       float default_inner_level[2];
@@ -1394,7 +1392,6 @@ void iris_finalize_program(struct iris_compiled_shader *shader,
                            uint32_t *streamout,
                            uint32_t *system_values,
                            unsigned num_system_values,
-                           unsigned kernel_input_size,
                            unsigned num_cbufs,
                            const struct iris_binding_table *bt);
 
@@ -1483,7 +1480,7 @@ int iris_get_driver_query_group_info(struct pipe_screen *pscreen,
 void gfx9_toggle_preemption(struct iris_context *ice,
                             struct iris_batch *batch,
                             const struct pipe_draw_info *draw);
-static const bool
+static bool
 iris_execute_indirect_draw_supported(const struct iris_context *ice,
                                      const struct pipe_draw_indirect_info *indirect,
                                      const struct pipe_draw_info *draw)
@@ -1491,7 +1488,6 @@ iris_execute_indirect_draw_supported(const struct iris_context *ice,
    const struct iris_screen *screen = (struct iris_screen *)ice->ctx.screen;
    const struct iris_vs_data *vs_data =
       iris_vs_data(ice->shaders.prog[MESA_SHADER_VERTEX]);
-   const bool is_multiview = draw->view_mask != 0;
    const size_t struct_size = draw->index_size ?
       sizeof(uint32_t) * 5 :
       sizeof(uint32_t) * 4;
@@ -1502,7 +1498,6 @@ iris_execute_indirect_draw_supported(const struct iris_context *ice,
            aligned_stride &&
            (indirect &&
            !indirect->count_from_stream_output) &&
-           !is_multiview &&
            !(vs_data->uses_firstvertex ||
              vs_data->uses_baseinstance ||
              vs_data->uses_drawid));
@@ -1511,9 +1506,11 @@ iris_execute_indirect_draw_supported(const struct iris_context *ice,
 #ifdef genX
 #  include "iris_genx_protos.h"
 #else
-#  define genX(x) gfx8_##x
-#  include "iris_genx_protos.h"
-#  undef genX
+#  ifdef INTEL_USE_ELK
+#    define genX(x) gfx8_##x
+#    include "iris_genx_protos.h"
+#    undef genX
+#  endif
 #  define genX(x) gfx9_##x
 #  include "iris_genx_protos.h"
 #  undef genX
@@ -1529,6 +1526,9 @@ iris_execute_indirect_draw_supported(const struct iris_context *ice,
 #  define genX(x) gfx20_##x
 #  include "iris_genx_protos.h"
 #  undef genX
+#  define genX(x) gfx30_##x
+#  include "iris_genx_protos.h"
+#  undef genX
 #endif
 
 #endif
