@@ -210,6 +210,7 @@ destroy_cmd_buffer(struct anv_cmd_buffer *cmd_buffer)
 
    while (u_vector_length(&cmd_buffer->dynamic_bos) > 0) {
       struct anv_bo **bo = u_vector_remove(&cmd_buffer->dynamic_bos);
+      ANV_DMR_BO_FREE(&cmd_buffer->vk.base, *bo);
       anv_bo_pool_free((*bo)->map != NULL ?
                        &cmd_buffer->device->batch_bo_pool :
                        &cmd_buffer->device->bvh_bo_pool, *bo);
@@ -458,6 +459,7 @@ anv_cmd_buffer_set_ray_query_buffer(struct anv_cmd_buffer *cmd_buffer,
                                                ANV_BO_ALLOC_INTERNAL, /* alloc_flags */
                                                0, /* explicit_address */
                                                &new_bo);
+         ANV_DMR_BO_ALLOC(&cmd_buffer->vk.base, new_bo, result);
          if (result != VK_SUCCESS) {
             anv_batch_set_error(&cmd_buffer->batch, result);
             return;
@@ -465,6 +467,7 @@ anv_cmd_buffer_set_ray_query_buffer(struct anv_cmd_buffer *cmd_buffer,
 
          bo = p_atomic_cmpxchg(&device->ray_query_shadow_bos[idx][bucket], NULL, new_bo);
          if (bo != NULL) {
+            ANV_DMR_BO_FREE(&device->vk.base, new_bo);
             anv_device_release_bo(device, new_bo);
          } else {
             bo = new_bo;
@@ -551,6 +554,7 @@ anv_cmd_buffer_flush_pipeline_state(struct anv_cmd_buffer *cmd_buffer,
    diff_fix_state(VF_SGVS,                  final.vf_sgvs);
    if (cmd_buffer->device->info->ver >= 11)
       diff_fix_state(VF_SGVS_2,             final.vf_sgvs_2);
+   diff_fix_state(VF_COMPONENT_PACKING,     final.vf_component_packing);
    if (cmd_buffer->device->info->ver >= 12)
       diff_fix_state(PRIMITIVE_REPLICATION, final.primitive_replication);
    diff_fix_state(SBE,                      final.sbe);
@@ -742,12 +746,7 @@ anv_cmd_buffer_get_pipeline_layout_state(struct anv_cmd_buffer *cmd_buffer,
       return &cmd_buffer->state.compute.base;
 
    case VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR:
-      *out_stages &= VK_SHADER_STAGE_RAYGEN_BIT_KHR |
-         VK_SHADER_STAGE_ANY_HIT_BIT_KHR |
-         VK_SHADER_STAGE_CLOSEST_HIT_BIT_KHR |
-         VK_SHADER_STAGE_MISS_BIT_KHR |
-         VK_SHADER_STAGE_INTERSECTION_BIT_KHR |
-         VK_SHADER_STAGE_CALLABLE_BIT_KHR;
+      *out_stages &= ANV_RT_STAGE_BITS;
       return &cmd_buffer->state.rt.base;
 
    default:
@@ -837,12 +836,7 @@ anv_cmd_buffer_bind_descriptor_set(struct anv_cmd_buffer *cmd_buffer,
             !cmd_buffer->device->physical->indirect_descriptors ||
             (stages & (VK_SHADER_STAGE_TASK_BIT_EXT |
                        VK_SHADER_STAGE_MESH_BIT_EXT |
-                       VK_SHADER_STAGE_RAYGEN_BIT_KHR |
-                       VK_SHADER_STAGE_ANY_HIT_BIT_KHR |
-                       VK_SHADER_STAGE_CLOSEST_HIT_BIT_KHR |
-                       VK_SHADER_STAGE_MISS_BIT_KHR |
-                       VK_SHADER_STAGE_INTERSECTION_BIT_KHR |
-                       VK_SHADER_STAGE_CALLABLE_BIT_KHR));
+                       ANV_RT_STAGE_BITS));
 
          if (update_desc_sets) {
             struct anv_push_constants *push = &pipe_state->push_constants;
@@ -915,14 +909,6 @@ anv_cmd_buffer_bind_descriptor_set(struct anv_cmd_buffer *cmd_buffer,
     VK_SHADER_STAGE_MESH_BIT_EXT | \
     VK_SHADER_STAGE_TASK_BIT_EXT)
 
-#define ANV_RT_STAGE_BITS \
-   (VK_SHADER_STAGE_RAYGEN_BIT_KHR | \
-    VK_SHADER_STAGE_ANY_HIT_BIT_KHR | \
-    VK_SHADER_STAGE_CLOSEST_HIT_BIT_KHR | \
-    VK_SHADER_STAGE_MISS_BIT_KHR | \
-    VK_SHADER_STAGE_INTERSECTION_BIT_KHR | \
-    VK_SHADER_STAGE_CALLABLE_BIT_KHR)
-
 void anv_CmdBindDescriptorSets2KHR(
     VkCommandBuffer                             commandBuffer,
     const VkBindDescriptorSetsInfoKHR*          pInfo)
@@ -1102,15 +1088,15 @@ void anv_CmdBindVertexBuffers2(
       ANV_FROM_HANDLE(anv_buffer, buffer, pBuffers[i]);
 
       if (buffer == NULL) {
-         vb[firstBinding + i] = (struct anv_vertex_binding) {
-            .buffer = NULL,
-         };
+         vb[firstBinding + i] = (struct anv_vertex_binding) { 0 };
       } else {
          vb[firstBinding + i] = (struct anv_vertex_binding) {
-            .buffer = buffer,
-            .offset = pOffsets[i],
+            .addr = anv_address_physical(
+               anv_address_add(buffer->address, pOffsets[i])),
             .size = vk_buffer_range(&buffer->vk, pOffsets[i],
                                     pSizes ? pSizes[i] : VK_WHOLE_SIZE),
+            .mocs = anv_mocs(cmd_buffer->device, buffer->address.bo,
+                             ISL_SURF_USAGE_VERTEX_BUFFER_BIT),
          };
       }
       cmd_buffer->state.gfx.vb_dirty |= 1 << (firstBinding + i);
@@ -1122,6 +1108,36 @@ void anv_CmdBindVertexBuffers2(
    }
 }
 
+void anv_CmdBindIndexBuffer2KHR(
+    VkCommandBuffer                             commandBuffer,
+    VkBuffer                                    _buffer,
+    VkDeviceSize                                offset,
+    VkDeviceSize                                size,
+    VkIndexType                                 indexType)
+{
+   ANV_FROM_HANDLE(anv_cmd_buffer, cmd_buffer, commandBuffer);
+   ANV_FROM_HANDLE(anv_buffer, buffer, _buffer);
+
+   if (cmd_buffer->state.gfx.index_type != indexType) {
+      cmd_buffer->state.gfx.index_type = indexType;
+      cmd_buffer->state.gfx.dirty |= ANV_CMD_DIRTY_INDEX_TYPE;
+   }
+
+   uint64_t index_addr = buffer ?
+      anv_address_physical(anv_address_add(buffer->address, offset)) : 0;
+   uint32_t index_size = buffer ? vk_buffer_range(&buffer->vk, offset, size) : 0;
+   if (cmd_buffer->state.gfx.index_addr != index_addr ||
+       cmd_buffer->state.gfx.index_size != index_size) {
+      cmd_buffer->state.gfx.index_addr = index_addr;
+      cmd_buffer->state.gfx.index_size = index_size;
+      cmd_buffer->state.gfx.index_mocs =
+         anv_mocs(cmd_buffer->device, buffer->address.bo,
+                  ISL_SURF_USAGE_INDEX_BUFFER_BIT);
+      cmd_buffer->state.gfx.dirty |= ANV_CMD_DIRTY_INDEX_BUFFER;
+   }
+}
+
+
 void anv_CmdBindTransformFeedbackBuffersEXT(
     VkCommandBuffer                             commandBuffer,
     uint32_t                                    firstBinding,
@@ -1139,14 +1155,17 @@ void anv_CmdBindTransformFeedbackBuffersEXT(
    assert(firstBinding + bindingCount <= MAX_XFB_BUFFERS);
    for (uint32_t i = 0; i < bindingCount; i++) {
       if (pBuffers[i] == VK_NULL_HANDLE) {
-         xfb[firstBinding + i].buffer = NULL;
+         xfb[firstBinding + i] = (struct anv_xfb_binding) { 0 };
       } else {
          ANV_FROM_HANDLE(anv_buffer, buffer, pBuffers[i]);
-         xfb[firstBinding + i].buffer = buffer;
-         xfb[firstBinding + i].offset = pOffsets[i];
-         xfb[firstBinding + i].size =
-            vk_buffer_range(&buffer->vk, pOffsets[i],
-                            pSizes ? pSizes[i] : VK_WHOLE_SIZE);
+         xfb[firstBinding + i] = (struct anv_xfb_binding) {
+            .addr = anv_address_physical(
+               anv_address_add(buffer->address, pOffsets[i])),
+            .size = vk_buffer_range(&buffer->vk, pOffsets[i],
+                                    pSizes ? pSizes[i] : VK_WHOLE_SIZE),
+            .mocs = anv_mocs(cmd_buffer->device, buffer->address.bo,
+                             ISL_SURF_USAGE_STREAM_OUT_BIT),
+         };
       }
    }
 }
@@ -1211,6 +1230,15 @@ anv_cmd_buffer_gfx_push_constants(struct anv_cmd_buffer *cmd_buffer)
    const struct anv_push_constants *data =
       &cmd_buffer->state.gfx.base.push_constants;
 
+   /* For Mesh/Task shaders the 3DSTATE_(MESH|TASK)_SHADER_DATA require a 64B
+    * alignment.
+    *
+    * ATMS PRMs Volume 2d: Command Reference: Structures,
+    * 3DSTATE_MESH_SHADER_DATA_BODY::Indirect Data Start Address:
+    *
+    *    "This pointer is relative to the General State Base Address. It is
+    *     the 64-byte aligned address of the indirect data."
+    */
    struct anv_state state =
       anv_cmd_buffer_alloc_temporary_state(cmd_buffer,
                                            sizeof(struct anv_push_constants),
@@ -1250,9 +1278,9 @@ anv_cmd_buffer_cs_push_constants(struct anv_cmd_buffer *cmd_buffer)
       ALIGN(total_push_constants_size, push_constant_alignment);
    struct anv_state state;
    if (devinfo->verx10 >= 125) {
-      state = anv_state_stream_alloc(&cmd_buffer->general_state_stream,
-                                     aligned_total_push_constants_size,
-                                     push_constant_alignment);
+      state = anv_cmd_buffer_alloc_general_state(cmd_buffer,
+                                                 aligned_total_push_constants_size,
+                                                 push_constant_alignment);
    } else {
       state = anv_cmd_buffer_alloc_dynamic_state(cmd_buffer,
                                                  aligned_total_push_constants_size,
@@ -1453,6 +1481,7 @@ void anv_CmdSetRayTracingPipelineStackSizeKHR(
                                             ANV_BO_ALLOC_INTERNAL, /* alloc_flags */
                                             0, /* explicit_address */
                                             &new_bo);
+      ANV_DMR_BO_ALLOC(&device->vk.base, new_bo, result);
       if (result != VK_SUCCESS) {
          rt->scratch.layout.total_size = 0;
          anv_batch_set_error(&cmd_buffer->batch, result);
@@ -1461,7 +1490,8 @@ void anv_CmdSetRayTracingPipelineStackSizeKHR(
 
       bo = p_atomic_cmpxchg(&device->rt_scratch_bos[bucket], NULL, new_bo);
       if (bo != NULL) {
-         anv_device_release_bo(device, bo);
+         ANV_DMR_BO_FREE(&device->vk.base, new_bo);
+         anv_device_release_bo(device, new_bo);
       } else {
          bo = new_bo;
       }
