@@ -965,7 +965,7 @@ impl AssignRegsBlock {
     }
 
     fn try_coalesce(&mut self, ssa: SSAValue, src: &Src) -> bool {
-        debug_assert!(src.src_mod.is_none());
+        debug_assert!(src.is_unmodified());
         let SrcRef::Reg(src_reg) = src.src_ref else {
             return false;
         };
@@ -1018,7 +1018,7 @@ impl AssignRegsBlock {
             }
             Op::PhiSrcs(phi) => {
                 for (id, src) in phi.srcs.iter() {
-                    assert!(src.src_mod.is_none());
+                    assert!(src.is_unmodified());
                     if let Some(ssa) = src_ssa_ref(src) {
                         assert!(ssa.comps() == 1);
                         let reg = self.get_scalar(ssa[0]);
@@ -1387,15 +1387,17 @@ impl AssignRegsBlock {
             pcopy.push(dst.into(), src.into());
         }
 
-        let ann = OpAnnotate {
-            annotation: "generated by assign_regs".into(),
-        };
-        if b.branch().is_some() {
-            b.instrs.insert(b.instrs.len() - 1, Instr::new_boxed(ann));
-            b.instrs.insert(b.instrs.len() - 1, Instr::new_boxed(pcopy));
-        } else {
-            b.instrs.push(Instr::new_boxed(ann));
-            b.instrs.push(Instr::new_boxed(pcopy));
+        if !pcopy.is_empty() {
+            let ann = OpAnnotate {
+                annotation: "generated by assign_regs".into(),
+            };
+            if b.branch().is_some() {
+                b.instrs.insert(b.instrs.len() - 1, Instr::new_boxed(ann));
+                b.instrs.insert(b.instrs.len() - 1, Instr::new_boxed(pcopy));
+            } else {
+                b.instrs.push(Instr::new_boxed(ann));
+                b.instrs.push(Instr::new_boxed(pcopy));
+            }
         }
     }
 }
@@ -1419,7 +1421,7 @@ impl Shader<'_> {
         for file in spill_files {
             let num_regs = self.sm.num_regs(file);
             if max_live[file] > num_regs {
-                f.spill_values(file, num_regs);
+                f.spill_values(file, num_regs, &mut self.info);
 
                 // Re-calculate liveness after we spill
                 live = SimpleLiveness::for_function(f);
@@ -1437,13 +1439,32 @@ impl Shader<'_> {
         let mut gpr_limit = max(max_live[RegFile::GPR], 16);
         let mut total_gprs = gpr_limit + u32::from(tmp_gprs);
 
-        let mut max_gprs = if DEBUG.spill() {
-            // We need at least 16 registers to satisfy RA constraints for
-            // texture ops and another 2 for parallel copy lowering
-            18
-        } else {
-            self.sm.num_regs(RegFile::GPR)
-        };
+        let mut max_gprs = self.sm.num_regs(RegFile::GPR);
+
+        if DEBUG.spill() {
+            // To test spilling, reduce the number of registers to the minimum
+            // practical for RA.  We need at least 16 registers to satisfy RA
+            // constraints for texture ops.
+            max_gprs = 16;
+
+            // OpRegOut can use arbitrarily many GPRs
+            for b in &f.blocks {
+                for instr in b.instrs.iter().rev() {
+                    match &instr.op {
+                        Op::Exit(_) => (),
+                        Op::RegOut(op) => {
+                            let out_gprs =
+                                u32::try_from(op.srcs.len()).unwrap();
+                            max_gprs = max(max_gprs, out_gprs);
+                        }
+                        _ => break,
+                    }
+                }
+            }
+
+            // and another 2 for parallel copy
+            max_gprs += 2;
+        }
 
         if let ShaderStageInfo::Compute(cs_info) = &self.info.stage {
             max_gprs = min(
@@ -1461,7 +1482,7 @@ impl Shader<'_> {
             total_gprs = max_gprs;
             gpr_limit = total_gprs - u32::from(tmp_gprs);
 
-            f.spill_values(RegFile::GPR, gpr_limit);
+            f.spill_values(RegFile::GPR, gpr_limit, &mut self.info);
 
             // Re-calculate liveness one last time
             live = SimpleLiveness::for_function(f);
